/**
 * Copyright (c) 2014,2018 by the respective copyright holders.
 *
 * See the NOTICE file(s) distributed with this work for additional
 * information regarding copyright ownership.
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0
 *
 * SPDX-License-Identifier: EPL-2.0
 */
package org.openhab.binding.openlicht.handler;

import static org.openhab.binding.openlicht.internal.BindingConstants.*;

import java.util.HashSet;
import java.util.Set;

import org.eclipse.jdt.annotation.Nullable;
import org.eclipse.smarthome.core.library.types.DecimalType;
import org.eclipse.smarthome.core.thing.ChannelUID;
import org.eclipse.smarthome.core.thing.Thing;
import org.eclipse.smarthome.core.types.Command;
import org.eclipse.smarthome.io.transport.mqtt.MqttService;
import org.openhab.binding.openlicht.internal.OpenLichtConfiguration;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * The {@link PolarHandler} is responsible for handling commands, which are
 * sent to one of the channels.
 *
 * @author René Schöne - Initial contribution
 */
public class PolarHandler extends AbstractMqttHandler {

    private final Logger logger = LoggerFactory.getLogger(PolarHandler.class);

    @Nullable
    private OpenLichtConfiguration config;

    private Set<String> seenUnsupportedCategories;

    public PolarHandler(Thing thing, MqttService mqttService) {
        super(thing, mqttService);
        seenUnsupportedCategories = new HashSet<>();
    }

    @Override
    public void handleCommand(ChannelUID channelUID, Command command) {
        logger.debug("Got command for read-only thing: {} {}", channelUID.getAsString(), command.toFullString());
        if (channelUID.getId().equals(CHANNEL_ACCELERATOR_X)) {
            // TODO: handle command

            // Note: if communication with thing fails for some reason,
            // indicate that by setting the status with detail information
            // updateStatus(ThingStatus.OFFLINE, ThingStatusDetail.COMMUNICATION_ERROR,
            // "Could not control device at IP address x.x.x.x");
        }
    }

    @Override
    public void processMessage(String topic, byte[] payload) {
        // switch category of message (last part of topic after last slash)
        String category = topic.substring(getTopicLength());
        String state = new String(payload);
        switch (category) {
            case MQTT_CATEGORY_BRIGHTNESS:
                updateState(CHANNEL_BRIGHTNESS_POLAR, DecimalType.valueOf(state));
                break;
            default:
                logUnsupportedCategory(category);
        }
    }

    private void logUnsupportedCategory(String category) {
        if (seenUnsupportedCategories.add(category)) {
            logger.warn("Category not supported: {}", category);

        }
    }

    @Override
    protected String getSubTopic() {
        return "polar";
    }

    @Override
    protected boolean subscribeSubTopics() {
        return true;
    }

}
